/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.core;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * PasswordService provides
 * 
 * @author Timo Westk&auml;mper
 * @version $Id: PasswordService.java 5487 2009-04-24 08:49:46Z tiwe $
 */
public final class PasswordService {
    private static final PasswordService instance = new PasswordService();

    private static final String HASH_ALGORITHM = "SHA-256";

    private static final String ENCODING = "UTF-8";

    private static final String CHARACTERS = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";

    private static final int DEFAULT_RANDOM_ID_LENGTH = 20;

    /**
     * Password must contain at least
     * <ul>
     * <li>8 characters,</li>
     * <li>one lower case letter,</li>
     * <li>one upper case letter and</li>
     * <li>one number (0-9) or one special character chosen from
     * "#$%&amp;'()*+,-./:;&lt;=&gt;?@[\]^_`{|}~</li>
     * </ul>
     */
    private static final String VALID_PASSWORD_REGEX =
    // "^[^\\s]*(?=.{8,})(?=.*\\d)(?=.*\\p{Lower})(?=.*\\p{Upper})(?=.*\\p{Punct})[^\\s]*$";
    "^.*(?=.{8,})(?=.*\\p{Lower})(?=.*\\p{Upper})(?=.*[\\d\\p{Punct}]).*$";

    private PasswordService() {
    }

    public synchronized String encrypt(String plaintext)
            throws SystemUnavailableException {
        MessageDigest md = null;
        try {
            md = MessageDigest.getInstance(HASH_ALGORITHM);
        } catch (NoSuchAlgorithmException e) {
            throw new SystemUnavailableException(e.getMessage());
        }
        try {
            md.update(plaintext.getBytes(ENCODING));
        } catch (UnsupportedEncodingException e) {
            throw new SystemUnavailableException(e.getMessage());
        }
        byte[] raw = md.digest();
        String hash = Base64.encode(raw);
        return hash;
    }

    public String getRandomId() {
        return getRandomId(DEFAULT_RANDOM_ID_LENGTH);
    }

    public String getRandomId(int length) {
        StringBuilder sb = new StringBuilder(length);
        for (int i = 0; i < length; i++) {
            int r = (int) (Math.random() * CHARACTERS.length());
            sb.append(CHARACTERS.charAt(r));
        }
        return sb.toString();
    }

    public static PasswordService getInstance() {
        return instance;
    }

    public static void main(String[] args) {
        if (args.length > 0) {
            try {
                System.out.println(getInstance().encrypt(args[0]));
            } catch (SystemUnavailableException e) {
                // TODO Error handling
                e.printStackTrace();
            }
        } else {
            System.out
                    .println("Usage: java com.mysema.util.PasswordService <password>\n"
                            + "Returns Base64-encoded SHA-1 hash of the given password.");
        }
    }

    /**
     * @param password
     * @return
     */
    public boolean validatePassword(String password) {
        return password != null && password.matches(VALID_PASSWORD_REGEX);
    }

}